function [testL trainL Ust filledYt] = approximate_test_likelihood(Yt, params, Y, w, factor_idx, ... 
    segments, MGPopts, K, invK, ldK, nmodeK, nmodeN, Kd, invKd, ldKd, initUs)

global L_BFGS OPT_MASK

missingData = 0; 

if (sum(sum(isnan(Yt)))) 
    missingData = 1; 
end

N = size(Y,1);
D = size(Y,2);
Nt = size(Yt,1);
nfactor = size(factor_idx,2); 
[Us, Hps, Phps, theta, beta] = fromOptimizerParams(params, MGPopts);

% [K, invK, nmodeK, nmodeN] = ...
%     computeNModeKernel(Us, Hps, factor_idx, theta, beta);

trainL = 0;
trainL = -D*N/2*log(2*pi)-D/2*ldK;
for d= 1:D
    trainL = trainL -0.5*w(d)*w(d)*Y(:, d)'*invK*Y(:, d);
end
trainL = trainL + N*sum(log(w));
fprintf(2, 'Training Likelihood %f\n', trainL);

% Kd = cell(nfactor);
% invKd = cell(nfactor);
allUs = cell(nfactor);
factor_idx = [factor_idx; zeros(Nt, nfactor)];
for n = 1:nfactor
%     % If GPDM prior, precompute the dynamics kernel.
%     if (~isempty(Phps{n}))
%         [Xin Xout] = priorIO(Us{n}, segments{n}, MGPopts{n}.priorModel);
%         [Kd{n} invKd{n}] = computePriorKernel(Xin, Phps{n}, MGPopts{n}.priorModel(3));
%     end
    
    if (exist('initUs', 'var'))
        allUs{n} = initUs{n};
    else
        lastfilledIdx = 0;
        % Initialize test data using NN.
        for i = 1:Nt
            if (sum(isnan(Yt(n,:))) == 0)
                [tmp idx] = min(sum((repmat(Yt(n,:), N, 1) - Y).^2,2));
                allUs{n}(i,:) = Us{n}(idx,:);
                lastfilledIdx = idx; 
            else
                if (lastfilledIdx == 0) 
                    allUs{n}(i,:) = zeros(1,D);
                else
                    allUs{n}(i,:) = Us{n}(idx,:); 
                end
            end
        end
    end
    
    matIdx{n} = [ones(size(Us{n})); zeros(size(allUs{n}))];
    allUs{n} = [Us{n}; allUs{n}];
    tseg = segments;
    tseg{n} = [segments{n} MGPopts{n}.Ns + 1];
    MGPopts{n}.Ns = MGPopts{n}.Ns + Nt;
    factor_idx(N+1:end,n) = [max(factor_idx(:,n)) + [1:Nt]']; 
end

params = toOptimizerParams(allUs, Hps, Phps, theta, beta);
OPT_MASK = [find(matIdx{1}(:)' > 0) length(matIdx{1}(:)') + [1:length([log(Hps{1}) log(Phps{1})])]];
for n = 2:nfactor
    nextm = [find(matIdx{n}(:)' > 0) length(matIdx{n}(:)') + [1:length([log(Hps{n}) log(Phps{n})])]];
    OPT_MASK = [OPT_MASK OPT_MASK(end) + nextm];
end
OPT_MASK = [OPT_MASK OPT_MASK(end) + [1:length([log(theta) log(beta)])]];
%             allUs{1}(end-1:end,:)
%             hold on;
%             plotseries(Us{1}, segments{1}, 'b');
%             plotseries(allUs{1}(end-1:end,:), [1], 'g');
%             pause;

optIdx = setdiff(1:length(params),OPT_MASK);
realparams = params(optIdx);
fn = @neglogprob_of_everything;

if L_BFGS == 1
    opts = lbfgs_options('iprint', 1, 'maxits', 100, 'cb', @test_callback);
    [realparams,fx,exitflag,userdata] = lbfgs(@(x)fn(x, params, [Y; Yt], w, factor_idx, tseg, MGPopts, K, invK, ldK, nmodeK, nmodeN, Kd, invKd, ldKd),realparams,opts);
    realparams = realparams';
else
    opts = optimset('Display','iter','GradObj','on','MaxIter', 100, 'LargeScale', 'off', 'DerivativeCheck', 'on');
    [realparams,fx,exitflag] = fminunc(@(x)fn(x, params, [Y; Yt], w, factor_idx, tseg, MGPopts, K, invK, ldK, nmodeK, nmodeN, Kd, invKd, ldKd),realparams,opts);
end

[f df filledY] = neglogprob_of_everything(realparams, params, [Y; Yt], w, factor_idx, tseg, MGPopts, K, invK, ldK, nmodeK, nmodeN, Kd, invKd, ldKd);

params(optIdx) = realparams;

[allUs, Hps, Phps, theta, beta] = fromOptimizerParams(params, MGPopts);

for n = 1:nfactor
    Ust{n} = allUs{n}(N+1:end,:);
end
filledYt = filledY(N+1:end,:);
testL = 0;
if (missingData == 0)
    [K, invK, ldK] = ...
        computeNModeKernel(allUs, Hps, factor_idx, theta, beta);

    allY = filledY;
    testL = -D*(size(allY,1))/2*log(2*pi)-D/2*ldK;
    for d= 1:D
        testL = testL -0.5*w(d)*w(d)*allY(:, d)'*invK*allY(:, d);
    end
    testL = testL + N*sum(log(w));
    testL = testL - trainL;
    fprintf(2, 'Test Likelihood %f\n', testL);
end
