function [Ysample Ymean Yvar] = ... 
    sampleNModeReconstruction(Us_star, Us, Y, idx, Hps, theta, beta, w, Kn, invKn) 
% Draw a random sample and returns the mean of the GP prediciton
% distribution given a MGP model.
%
% Us_star - new inputs
% Us - training inputs 
% Y - training outputs
% idx - factor indices for training data
% Hps, theta, beta, w - hyperparameters
% Kn, invKn - kernel matrix of the training data
% This only works for M == 1 for now. 

M = size(Us_star{1}, 1); 
N = size(Y,1); 
D = size(Y,2);
nIdx = [];
for i = 1:size(idx,2)
    allUs{i} = [Us{i}; Us_star{i}];
    nIdx = [nIdx  [1:M]']; 
end

numModes = size(idx,2); 
Ka = ones(N, M);
Kb = ones(M, M);
nmodeK = cell(numModes, 1);
for n = 1:numModes
    U = allUs{n}(1:size(Us{n},1),:);
    V = allUs{n}(size(Us{n},1)+1:end,:);
    gamma = Hps{n};  

    if isempty(gamma)
        Kna = U*V'; 
        Knb = V*V'; 
    else
        N2 = dist2(U, V);
        Kna = exp(-N2*gamma/2);
        N2 = dist2(V, V); 
        Knb = exp(-N2*gamma/2); 
    end
    nmodeKa{n} = Kna(idx(:,n), nIdx(:,n)); 
    Ka = Ka.*nmodeKa{n};
    nmodeKb{n} = Knb(nIdx(:,n), nIdx(:,n)); 
    Kb = Kb.*nmodeKb{n};
end
A = theta*Ka; 
B = theta*Kb + eye(M)*1/beta;

Km = B - A'*invKn*A; 

mu = []; 
C = []; 
Ymean = zeros(M,D); 
Ysample = zeros(M,D);
rootKm = Km^(1/2); 
z = randn(1,1);
tmp = A'*invKn; 
for q = 1:D
    Ymean(M,q) = tmp*Y(:,q); 
    Ysample(M,q) = (1/w(q))*rootKm*z + Ymean(1,q); 
end
Yvar = rootKm^2; 
