addpath ../gpdm/src/netlab ../gpdm/src/gpdm ../gpdm/src/gplvm ... 
    ../gpdm ../gpdm/src/util ./lbfgs ./LHSutils 
dataDir = '../gpdm/';

format long; 

global OPT_MASK L_BFGS;  

% Define optimizer to use
L_BFGS = 0; 

% Define the type of model to learn
% MODEL == 2 && INIT_CIRCLE == 1 means circular dyn. model from ICML 07'
% MODEL == 1 && INIT_CIRCLE == 0 m&& NOSTYLE == 1 means GPDM
MODEL = 1;  
NOSTYLE = 1;

% How to initialize the content factor, set to one if MODEL == 2
INIT_CIRCLE = 0; 

% Define global variables
WVariance = 0; 

% Load data
[Y initY segments{1}] = loadMocapData({ ... 
    [dataDir, '/mocaps/02/02_02.amc'], ...
    [dataDir, '/mocaps/02/02_03.amc'], ... 
    [dataDir, '/mocaps/35/35_01.amc'], ...     
    [dataDir, '/mocaps/35/35_18.amc'], ...
    [dataDir, '/mocaps/08/08_01.amc'], ...
    [dataDir, '/mocaps/08/08_07.amc'],  ...
    }, ...
    [1 1 1 1 1 1], 4*ones(1,6), [200 -1 200 -1 200 -1]);

refY = Y; 
N = size(Y,1); 
D = size(Y,2); 

% Label the identity-gait factors for each segment, and save the segments 
% as individual .amc files
segmentLabel = [1 1; 1 2; 2 1; 2 2; 3 1; 3 3]; 
for i=1:length(segments{1})-1
    factor_idx(segments{1}(i):segments{1}(i+1)-1,2:3) = ... 
        repmat(segmentLabel(i,:),segments{1}(i+1)-segments{1}(i),1); 
%     saveMocapData( strcat('training_data_',mat2str(i),'.amc'), ... 
%         refY(segments{1}(i):segments{1}(i+1)-1,:),initY(i,:));
end
factor_idx(segments{1}(end):N,2:3) = ... 
     repmat(segmentLabel(end,:),N-segments{1}(end)+1,1); 
% saveMocapData( strcat('training_data_',mat2str(length(segments{1})),'.amc'), ... 
%     refY(segments{1}(end):end,:),initY(length(segments{1}),:));
factor_idx(:,1) = [1:N]';

% Mean-substract the data, normalize the variance in each dimension
meanData = mean(Y);
Y = Y - repmat(meanData, N, 1);
vY = var(Y); 
for d = 1:D
    Y(:,d) = Y(:,d)/sqrt(vY(d)); 
end

if (MODEL == 1)
    % GPDM
    % priorModel{i}(1) : input of dynamics
    %   0 => [x_t, x_{t-1}]
    %   1 => [x_t, x_t - x_{t-1}]
    %   2 => [x_t]
    % priorModel{i}(2) : output of dynamics
    %   0 => x_{t+1}
    %   1 => x_{t+1} - x_t
    % priorModel{i}(3) : kernel type
    %   -1 => Isotropic Gaussian
    %   0 => RBF kernel with weighted dimensions, use with input 0 or 1
    %   1 => RBF kernel
    %   2 => Linear kernel
    %   3 => weighted Linear kernel + RBF kernel with weighted dimensions, use with
    %   input 0 or 1
    %   4 => weighted linear kernel
    %   5 => linear + RBF
    % 
    % priorModel is only used if Phps is non-empty
    
    MGPopts{1} = struct('Ns', N, 'qs', 3, 'Nhps', 1, 'Nphps', 4, 'priorModel', [2 0 5], ...
        'periodic', 0, 'covS', [], 'meanS', [], 'G', [], 'BalanceTerm', 1);
    MGPopts{2} = struct('Ns', 1, 'qs', 1, 'Nhps', 0, 'Nphps', 0, 'priorModel', [], ...
        'periodic', 0, 'covS', [], 'meanS', [], 'G', [], 'BalanceTerm', 1);
    MGPopts{3} = struct('Ns', 1, 'qs', 1, 'Nhps', 0, 'Nphps', 0, 'priorModel', [], ...
        'periodic', 0, 'covS', [], 'meanS', [], 'G', [], 'BalanceTerm', 1);

    Hps = cell(1,3);
    Phps = cell(1,3);
    Hps{1} = 1;
    Phps{1} = [0.9 1 0.1 exp(1)];
elseif (MODEL == 2) 
    % Circle
    MGPopts{1} = struct('Ns', N, 'qs', 2, 'Nhps', 1, 'Nphps', 0, 'priorModel', [], ...
        'periodic', 2, 'covS', [], 'meanS', [], 'G', [], 'BalanceTerm', 1);
    MGPopts{2} = struct('Ns', 1, 'qs', 1, 'Nhps', 0, 'Nphps', 0, 'priorModel', [], ...
        'periodic', 0, 'covS', [], 'meanS', [], 'G', [], 'BalanceTerm', 1);
    MGPopts{3} = struct('Ns', 1, 'qs', 1, 'Nhps', 0, 'Nphps', 0, 'priorModel', [], ...
        'periodic', 0, 'covS', [], 'meanS', [], 'G', [], 'BalanceTerm', 1);
    
    Hps = cell(1,3);
    Phps = cell(1,3);
    Hps{1} = 1;
    Phps{1} = []; 
end

if (NOSTYLE == 1)
    % This reduces to regular GPLVM or GPDM
    factor_idx(:,2) = ones(N,1);
    factor_idx(:,3) = ones(N,1);
else
    MGPopts{2}.Ns = 3; 
    MGPopts{3}.Ns = 3; 
    MGPopts{2}.qs = 3; 
    MGPopts{3}.qs = 3; 
end

% For notational convenience
for n = 1:length(MGPopts)
    Ns(n) = MGPopts{n}.Ns; 
    qs(n) = MGPopts{n}.qs; 
end

% Initialize hyperparameters
theta = 1; 
beta = exp(1); 

% Initialize content using circle or PCA
if INIT_CIRCLE == 1
    t{1} = initCircle(Y, segments{1});
    %Rx = [1 0 0; 0 cos(pi/4) sin(pi/4); 0 -sin(pi/4) cos(pi/4)];
    Us{1} = [toCircle(t{1}, segments{1}, Ns(1)) 0.1*ones(Ns(1),qs(1)-2)];
else
    [v, u] = pca(Y);
    v(find(v<0))=0;
    Us{1} = Y*u(:, 1:qs(1))*diag(1./sqrt(v(1:qs(1))));
end

% Initialize styles to identity matrix
Us{2} = eye(Ns(2), qs(2)); 
Us{3} = eye(Ns(3), qs(3)); 

extItr = 10; 
inItr = 100; 
for iters = 1:extItr
    fprintf(2,'Iteration %d\n',iters);

    [K, invK] = ... 
        computeNModeKernel(Us, Hps, factor_idx, theta, beta); 

    % Update scales every inItr iterations
    for d=1:D
        denom = Y(:,d)'*invK*Y(:,d); 
        if (WVariance > 0) 
            denom = denom + 1/WVariance; 
        end
        w(d) = sqrt(N/denom);
    end
    
%     If use nonzero mean Gaussian priors for styles    
%     if (NOSTYLE == 0)
%         meanS{2} = mean(Us{2});
%         meanS{3} = mean(Us{3});
%         covS{2} = eye(qs(2));
%         covS{3} = eye(qs(3));
%     end
%    

    if MODEL == 2
        % Setup optimization for circle parameterization
        params = [t{1} log(Hps{1}) [] Us{2}(:)' [] [] Us{3}(:)' [] [] log(theta) log(beta)];
        fidx = length(segments{1}) + 1;
        bidx = fidx + length(segments{1}) - 1;
        lb = repmat(-Inf,1,length(params));
        ub = repmat(Inf,1,length(params));
        lb(fidx:bidx) = zeros(1,length(segments{1})) + 1e-6;
        nbd = zeros(size(params));
        nbd(fidx:bidx) = ones(1,length(segments{1}));
        if NOSTYLE == 1
            % Don't optimize style inputs if we simulate old GPLVM
            OPT_MASK = length([t{1} log(Hps{1})]) + [1:2];
        end
    else
        params = toOptimizerParams(Us, Hps, Phps, theta, beta); 
        if NOSTYLE == 1
            % Don't optimize style inputs if we simulate old GPLVM
            OPT_MASK = length([Us{1}(:)' log(Hps{1}) log(Phps{1})]) + [1:2];
        end
    end
    
    % Do not optimize the overall kernel scale, since we already have the
    % w, but comment this out if w is not optimized.
    OPT_MASK = [OPT_MASK length(params)-1]; 
    
    optIdx = setdiff(1:length(params),OPT_MASK); 
    realparams = params(optIdx);
    
    
    fn = @neglogprob_of_everything;
    if L_BFGS == 1
        %gradchek_lbfgs(params, fn);
        opts = lbfgs_options('iprint', 1, 'maxits', inItr, 'cb', @test_callback);
    else
        opts = optimset('Display','iter','GradObj','on','MaxIter', inItr, 'LargeScale', 'off', 'DerivativeCheck', 'on');
    end

    if MODEL == 2
        if L_BFGS == 1
            [realparams,fx,exitflag,userdata] = lbfgs(@(x)fn(x, params, Y, w, ... 
                factor_idx, segments, MGPopts),realparams,lb,ub,nbd,opts);
            realparams = realparams';
        else
            [realparams,fx,exitflag] = fminunc(@(x)fn(x, params, Y, w, factor_idx, ... 
                segments, MGPopts),realparams,opts);
        end
        
        params(optIdx) = realparams; 

        currentIdx = 0;
        t{1} = params([1:2*length(segments{1})] + currentIdx);
        currentIdx = currentIdx + 2*length(segments{1});
        Hps{1} = exp(params([1:MGPopts{1}.Nhps] + currentIdx));
        currentIdx = currentIdx + MGPopts{1}.Nhps;
        Us{1} = toCircle(t{1}, segments{1}, Ns(1));
        Us{2} = reshape(params([1:Ns(2)*qs(2)] + currentIdx), Ns(2), qs(2));
        currentIdx = currentIdx + Ns(2)*qs(2);
        Us{3} = reshape(params([1:Ns(3)*qs(3)] + currentIdx), Ns(3), qs(3));
        currentIdx = currentIdx + Ns(3)*qs(3);
        theta = exp(params(end-1));
        beta = exp(params(end));
    else
        if L_BFGS == 1
            [realparams,fx,exitflag,userdata] = lbfgs(@(x)fn(x, params, Y, w, factor_idx, segments, MGPopts),realparams,opts);
            realparams = realparams';
        else
            [realparams,fx,exitflag] = fminunc(@(x)fn(x, params, Y, w, factor_idx, segments, MGPopts),realparams,opts);
        end
        
        params(optIdx) = realparams; 
          
        [Us, Hps, Phps, theta, beta] = fromOptimizerParams(params, MGPopts); 
    end

   
 
end
% If use nonzero mean Gaussian priors for styles    
% MGPopts{2}.meanS = mean(Us{2});
% MGPopts{2}.covS = cov(Us{2},1);
% 
% MGPopts{3} = mean(Us{3});
% MGPopts{3} = cov(Us{3},1);

% Save learned model
if MGPopts{1}.periodic == 2
    save lastMGPModel Us Y factor_idx w Hps Phps theta beta N D meanData  ...
        segments initY refY vY t segmentLabel MGPopts;
else
    save lastMGPModel Us Y factor_idx w Hps Phps theta beta N D meanData  ...
        segments initY refY vY segmentLabel MGPopts;
end






